!> code for the numerical solution of -\Delta u = f
!> with non-homogeneous Dirichlet BC using conforming P1 FEM
program FEM
  use o_mesh
  use o_matrix
  use o_sol
  
  implicit none
  real, dimension(:,:), allocatable :: xp
  integer, dimension(:,:), allocatable :: nodes, lbn
  integer, dimension(:,:), allocatable :: ip 
  real, dimension(:,:), allocatable :: mS, A
  real, dimension(:), allocatable :: u, v, f
  type(mesh), target :: grid
  character(len=1):: sparse
  real :: t1, t2
  integer :: npoin, ipoin
  integer :: itri, i, j, isparse


  if (command_argument_count() == 1) then
      call get_command_argument(1,sparse)
      if(sparse == 'S') then
         print*,'sparse variant'
      else
         print*,'dense variant'
      endif
   else
      print*,'Syntax  ./femP1 <S/D>'
      print*,' <S> .. sparse variant'
      print*,' <D> .. dense variant'
      stop
   endif
   
   call cpu_time( t1 )

  
  ! reading of the triangular mesh
  call Read_mesh("triang", grid, sparse) 
  npoin = grid%npoin
  ipoin = grid%ipoin

   if(sparse == 'S') then
      ! evaluation of the stiff matrix for interior and boundary nodes - SPARCE format
      call STIFF_MATRIX_SPARSE(grid )

   else
      ! stiff matrix
      allocate(mS(1:npoin, 1:npoin), source = 0.0 )

      ! evaluation of the stiff matrix for interior and boundary nodes
      call STIFF_MATRIX(grid,  mS(1:npoin, 1:npoin)  )
   endif
   

  !do i=1,npoin
  !   write(*,'(a6, i5, 2es12.4, 2i5, es12.4)') 'node:', i, grid%xp(i,1:2), grid%ip(i,1:2),grid%b_edge(i)
  !enddo

  ! assembling of the smaller matrix, including only interior node
   if(sparse /= 'S') then
      allocate(A(1:ipoin, 1:ipoin) )
      call Assemble_small_matrix(grid,  mS(1:npoin, 1:npoin), A(1:ipoin, 1:ipoin) )
   endif
  
!WR  do i=1,npoin
!WR     write(*,'(2i5,es12.4, a2, 300es12.4)') i, grid%ip(i, 1), sum(mS(i, :)),'|', mS(i, :)
!WR  enddo

  !write(*,'(a8,300i5)') 'irow:', grid%irow(:)
!WR  write(*,'(a8,300es12.4)') 'Msparse:', grid%sparse(:)

  
  !do i=1,ipoin
  !   write(*,'(2i5,es12.4, a2, 300es12.4)') i, grid%ip(i,2), sum(A(i, :)),'|', A(i, :)
  !enddo


  ! solution at interior and boundary nodes
  allocate(u (1:npoin) )

  ! right-hand side and the solution for algebraic problem, only in interior nodes
  allocate(f (1:ipoin), v (1:npoin) )

  
  ! evaluation of the RHS of the linar algebraic problem - SPARSE variant
  if(sparse == 'S') then
     call InitSol_RHS_Sparse(grid, u(1:npoin), v (1:npoin),f(1:ipoin))

  else
     ! evaluation of the RHS of the linar algebraic problem
     call InitSol_RHS(grid, mS(1:npoin, 1:npoin), u(1:npoin), v (1:npoin), f(1:ipoin) )
  endif

  if(sparse == 'S') then
     !iterative solution of SPARSE algebraic problem 
     call SolveMatrixProblem_Sparse(grid, ipoin, f(1:ipoin),v(1:ipoin)) 

  else
     ! iterative solution of smaller algebraic problem
     call SolveMatrixProblem(ipoin, A(1:ipoin, 1:ipoin), f(1:ipoin), v(1:ipoin) ) 
  endif

  ! association of the solution
  do i=1,npoin
     if(grid%ip(i,1) >= 0) u(i) = v(grid%ip(i,2) )
  enddo
  
  call cpu_time( t2 )

  write(*,'(a20,f8.1, a2)') 'Computed within ', t2-t1, ' s'

  ! drawing of the output solution
  call Plot_solution(grid, "result" , u(1:grid%npoin) )

  
  ! do i=1,npoin
  !    do j=1,npoin
  !       if(mS(i,j) /= 0.) then
  !          write(88, *) i,j

  !          if(i /=j) then
  !             write(92,*) (grid%xp(i, 1:2) + grid%xp(j, 1:2) ) / 2
  !          endif


  !       endif
  !    enddo
  !    write(91,*) grid%xp(i, 1:2) 
  ! enddo
  

  
end program FEM


  
